==> gwt-dev-2.8.20160530.jar: com/google/gwt/dev/jjs/ast/JCastOperation.java <==
/*
 * Copyright 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.dev.jjs.ast;

import com.google.gwt.dev.jjs.SourceInfo;

/**
 * Java cast expression.
 */
public class JCastOperation extends JExpression {

  private JType castType;
  private JExpression expr;

  public JCastOperation(SourceInfo info, JType castType, JExpression expr) {
    super(info);
    this.castType = castType;
    this.expr = expr;
  }

  public JType getCastType() {
    return castType;
  }

  public JExpression getExpr() {
    return expr;
  }

  @Override
  public JType getType() {
    if (!expr.getType().canBeNull() && !castType.isNullType()) {
      // Strengthen type to non null unless it has been determined that the type is not instantiable
      // (and that is reflected by replacing the cast type by the null type).
      return castType.strengthenToNonNull();
    }
    return castType;
  }

  @Override
  public boolean hasSideEffects() {
    // Any live cast operations might throw a ClassCastException
    //
    // TODO: revisit this when we support the concept of whether something
    // can/must complete normally!
    return true;
  }

  /**
   * Resolve an external reference during AST stitching.
   */
  public void resolve(JType newType) {
    assert newType.replaces(castType);
    castType = newType;
  }

  @Override
  public void traverse(JVisitor visitor, Context ctx) {
    if (visitor.visit(this, ctx)) {
      expr = visitor.accept(expr);
    }
    visitor.endVisit(this, ctx);
  }

}

==> gwt-dev-2.12.0.jar: com/google/gwt/dev/jjs/ast/JCastOperation.java <==
/*
 * Copyright 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.dev.jjs.ast;

import com.google.gwt.dev.jjs.SourceInfo;

/**
 * Java cast expression.
 */
public class JCastOperation extends JExpression {

  private JType castType;
  private JExpression expr;

  public JCastOperation(SourceInfo info, JType castType, JExpression expr) {
    super(info);
    this.castType = castType;
    this.expr = expr;
  }

  public JType getCastType() {
    return castType;
  }

  public JExpression getExpr() {
    return expr;
  }

  @Override
  public JType getType() {
    if (!expr.getType().canBeNull() && !castType.isNullType()) {
      // Strengthen type to non null unless it has been determined that the type is not instantiable
      // (and that is reflected by replacing the cast type by the null type).
      return castType.strengthenToNonNull();
    }
    return castType;
  }

  @Override
  public boolean hasSideEffects() {
    if (castType.isPrimitiveType()) {
      // Primitive casts do not throw and only have side effects if the expression has side effects.
      return expr.hasSideEffects();
    }
    // Any live non-primitive cast operations might throw a ClassCastException
    //
    // TODO: revisit this when we support the concept of whether something
    // can/must complete normally!
    return true;
  }

  /**
   * Resolve an external reference during AST stitching.
   */
  public void resolve(JType newType) {
    assert newType.replaces(castType);
    castType = newType;
  }

  @Override
  public void traverse(JVisitor visitor, Context ctx) {
    if (visitor.visit(this, ctx)) {
      expr = visitor.accept(expr);
    }
    visitor.endVisit(this, ctx);
  }

}
